@@ -58,6 +58,7 @@ int
 init_mem()
 {
 	if ((scratch = (token_type *) malloc(((n_tokens * 3) / 2) * sizeof(token_type))) == NULL
+	    || (tes = (token_type *) malloc(n_tokens * sizeof(token_type))) == NULL
 	    || (tlhs = (token_type *) malloc(n_tokens * sizeof(token_type))) == NULL
 	    || (trhs = (token_type *) malloc(n_tokens * sizeof(token_type))) == NULL) {
 		return false;
@@ -168,7 +169,7 @@ long	*vp;
 }
 
 /*
- * Allocate or reuse an equation space.
+ * Allocate or reuse an empty equation space.
  *
  * Returns new equation space number or -1 on error.
  */
@@ -222,6 +223,18 @@ next_espace()
 	return i;
 }
 
+/*
+ * Copy equation number "src" to equation number "dest".
+ */
+copy_espace(src, dest)
+int	src, dest;
+{
+	blt(lhs[dest], lhs[src], n_lhs[src] * sizeof(token_type));
+	n_lhs[dest] = n_lhs[src];
+	blt(rhs[dest], rhs[src], n_rhs[src] * sizeof(token_type));
+	n_rhs[dest] = n_rhs[src];
+}
+
 /*
  * Return true if equation number "i" is solved for a normal variable.
  */
@@ -592,7 +605,7 @@ int	en;	/* equation space number */
 #if	LIBRARY
 	if (gfp == stdout) {
 		if (display2d) {
-			group_sub(en);
+			make_fractions_and_group(en);
 		}
 		if (factor_int_flag) {
 			factor_int_sub(en);
