@@ -26,6 +26,7 @@
 #define	ARR_CNT(a)	(sizeof(a)/sizeof(a[0]))	/* returns the number of elements in an array */
 
 #define	MAX_K_INTEGER	1.0e14				/* maximum representable integer, 14 digits for doubles */
+#define	MAX_PRECISION	16				/* maximum useful display precision (number of digits) */
 
 #define	blt(dst, src, cnt)	memmove((char *) (dst), (char *) (src), (size_t) (cnt))	/* memory copy function */
 #define always_positive(power)	(fmod((double) (power), 2.0) == 0.0)	/* true if all real numbers raised to "power" result in positive, real numbers */
@@ -60,14 +61,17 @@
 
 /*
  * The following defines the default maximum mathematical expression size.
+ * Expression arrays are allocated with this size by default.
+ * It is linearly related to the actual memory usage of Mathomatic.
  * This should be made much smaller for Palmtops or embedded systems.
+ * Do not set to less than 100.
  */
 #define	DEFAULT_N_TOKENS	30000
 
 #define	DIVISOR_SIZE	(DEFAULT_N_TOKENS / 2)	/* a nice maximum divisor size */
 
 #define	MAX_VAR_NAMES	8000			/* maximum number of long variable names, keep this under (VAR_MASK - VAR_OFFSET) */
-#define	MAX_VAR_LEN	1000			/* maximum size of long variable names */
+#define	MAX_VAR_LEN	100			/* maximum size of long variable names */
 
 #define	MAX_VARS	500			/* maximum number of unique variables handled in each equation */
 
@@ -95,11 +99,8 @@ typedef union {
 #define	IMAGINARY	3L	/* the imaginary constant "i#" */
 #define	SIGN		4L	/* the "sign" variable */
 #define	MATCH_ANY	5L	/* match any variable (wild-card variable) */
-#define	V_ANSWER	6L	/* the "answer" variable */
-#define V_TEMP		7L	/* the "temp" variable */
-#define	V_INTEGER	8L	/* the "integer" variable */
 	int	operatr;	/* internal storage for mathematical operators */
-/* valid operators follow */
+/* valid operators follow (order doesn't matter) */
 #define	PLUS		1	/* a + b */
 #define	MINUS		2	/* a - b */
 #define	TIMES		3	/* a * b */
