@@ -291,18 +291,21 @@ int		en;		/* equation space number */
 	int	i, j, k;
 	int	level;
 	int	diff_sign;
-	int	found_se = true;
+	int	found_se;	/* found sub-expression flag */
 
-	for (level = 1; found_se; level++) {
+	if (!solved_equation(en)) {
+		return false;
+	}
+	for (level = 1, found_se = true; found_se; level++) {
 		for (i = 1, found_se = false; i < *np; i = j + 2) {
-			for (j = i; j < *np && equation[j].level > level; j += 2)
+			for (j = i; j < *np && equation[j].level >= level; j += 2)
 				;
 			if (j == i) {
 				continue;
 			}
 			found_se = true;
 			k = i - 1;
-			if ((j - k) >= OPT_MIN_SIZE && se_compare(&equation[k], j - k, rhs[en], n_rhs[en], &diff_sign)) {
+			if (se_compare(&equation[k], j - k, rhs[en], n_rhs[en], &diff_sign)) {
 				if (diff_sign) {
 					blt(&equation[i+2], &equation[j], (*np - j) * sizeof(token_type));
 					*np -= (j - (i + 2));
@@ -343,12 +346,11 @@ int		*np;
 	int	i, j, k, i1, i2, jj1, k1;
 	int	level, level1;
 	int	diff_sign;
-	int	found_se, found_se1;
+	int	found_se, found_se1;	/* found sub-expression flags */
 	long	v;
 	char	temp_buf[50];
 
-	found_se = true;
-	for (level = 1; found_se; level++) {
+	for (level = 1, found_se = true; found_se; level++) {
 		for (i = 1, found_se = false; i < *np; i = j + 2) {
 			for (j = i; j < *np && equation[j].level > level; j += 2)
 				;
@@ -440,27 +442,44 @@ char	*cp;
 	int	i, j, k, i1;
 	int	start, stop;
 	int	rv = false;
-	int	flag;
+	int	start_en;
 
 	if (!get_range_eol(&cp, &start, &stop)) {
 		return false;
 	}
+	opt_en[0] = -1;
+	start_en = 0;
+	for (i = start; i <= stop; i++) {
+		simp_sub(i);
+	}
+	for (i = start; i <= stop; i++) {
+		for (j = start; j <= stop; j++) {
+			if (i != j) {
+				while (find_more(rhs[i], &n_rhs[i], j)) {
+					rv = true;
+				}
+			}
+		}
+	}
 	for (i = start; i <= stop; i++) {
 		if (n_lhs[i] == 0)
 			continue;
-		opt_en[0] = -1;
-		simp_sub(i);
-		flag = false;
+		for (j = 0; opt_en[j] >= 0; j++) {
+			simp_sub(opt_en[j]);
+			while (find_more(lhs[i], &n_lhs[i], opt_en[j]))
+				;
+			while (find_more(rhs[i], &n_rhs[i], opt_en[j]))
+				;
+		}
 		while (opt_es(lhs[i], &n_lhs[i])) {
-			flag = true;
+			rv = true;
 		}
 		while (opt_es(rhs[i], &n_rhs[i])) {
-			flag = true;
-		}
-		if (flag) {
 			rv = true;
-			for (i1 = 0; opt_en[i1] >= 0; i1++) {
-				for (j = 0; opt_en[j] >= 0; j++) {
+		}
+		if (rv) {
+			for (i1 = start_en; opt_en[i1] >= 0; i1++) {
+				for (j = start_en; opt_en[j] >= 0; j++) {
 					for (k = j + 1; opt_en[k] >= 0; k++) {
 						while (find_more(rhs[opt_en[k]], &n_rhs[opt_en[k]], opt_en[j]))
 							;
@@ -471,8 +490,8 @@ char	*cp;
 				while (opt_es(rhs[opt_en[i1]], &n_rhs[opt_en[i1]]))
 					;
 			}
-			for (j = 0; opt_en[j] >= 0; j++) {
-				list_sub(opt_en[j]);
+			for (; opt_en[start_en] >= 0; start_en++) {
+				list_sub(opt_en[start_en]);
 			}
 			list_sub(i);
 		}
@@ -552,6 +571,11 @@ output_options()
 	}
 	fprintf(gfp, "autosolve\n");
 
+	if (!autocalc) {
+		fprintf(gfp, "no ");
+	}
+	fprintf(gfp, "autocalc\n");
+
 #if	!SILENT
 	fprintf(gfp, "debug_level = %d\n", debug_level);
 #endif
@@ -595,6 +619,8 @@ output_options()
 		fprintf(gfp, "no ");
 	}
 	fprintf(gfp, "factor_integers\n");
+
+	fprintf(gfp, "special_variable_character = %c\n", special_variable_character);
 }
 
 /*
@@ -661,6 +687,15 @@ next_option:
 		goto next_option;
 	}
 #endif
+	if (strncasecmp(cp, "special", 7) == 0) {
+		cp = skip_param(cp);
+		if (negate || *cp == '\0' || cp[1]) {
+			error(_("Please specify a single character."));
+			return false;
+		}
+		special_variable_character = *cp;
+		return true;
+	}
 	if (strncasecmp(cp, "columns", 7) == 0) {
 		cp = skip_param(cp);
 		if (negate) {
@@ -680,6 +715,11 @@ next_option:
 		autosolve = !negate;
 		goto next_option;
 	}
+	if (strncasecmp(cp, "autocalc", 8) == 0) {
+		cp = skip_param(cp);
+		autocalc = !negate;
+		goto next_option;
+	}
 	if (strncasecmp(cp, "case", 4) == 0) {
 		cp = skip_param(cp);
 		case_sensitive_flag = !negate;
@@ -774,10 +814,7 @@ char	*cp;
 	for (i1 = i; i1 <= j; i1++) {
 		if (exists[i1]) {
 			k = next_espace();
-			blt(lhs[k], lhs[i1], n_lhs[i1] * sizeof(token_type));
-			n_lhs[k] = n_lhs[i1];
-			blt(rhs[k], rhs[i1], n_rhs[i1] * sizeof(token_type));
-			n_rhs[k] = n_rhs[i1];
+			copy_espace(i1, k);
 			return_result(k);
 		}
 	}
@@ -833,14 +870,10 @@ int	imag_flag;	/* if true, copy the imaginary part, otherwise copy the real part
 	partial_flag = true;
 	n1 = 1;
 	dest[0] = zero_token;
-	has_imag = false;
-	has_real = false;
-	k = 0;
-	for (beg = k; beg < *nps; beg = k, k++) {
+	has_imag = has_real = false;
+	for (beg = k = 0; beg < *nps; beg = k, k++) {
 		found_imag = false;
-		for (;; k++) {
-			if (k >= *nps)
-				break;
+		for (; k < *nps; k++) {
 			if (source[k].level == 1 && source[k].kind == OPERATOR
 			    && (source[k].token.operatr == PLUS || source[k].token.operatr == MINUS)) {
 				break;
@@ -975,13 +1008,13 @@ char	*cp;
 {
 	int		i, j, k;
 	long		v, last_v, it_v = 0;
-	long		counter;
-	long		counter_max;
+	long		counter, counter_max;
 	sign_array_type	sa_mark;
 	sign_array_type	sa_value;
-	int		iterations = 1;
-	int		n;
+	long		l, iterations = 1;
 	token_type	*source;
+	int		n;
+	int		diff_sign;
 
 	if (current_not_defined()) {
 		return false;
@@ -1045,20 +1078,25 @@ char	*cp;
 	if (it_v) {
 		list_var(it_v, 0);
 		snprintf(prompt_str, sizeof(prompt_str), _("Enter initial %s: "), var_str);
-		while (!get_expr(scratch, &n))
+		while (!get_expr(tes, &n_tes))
 			;
+		calc_simp(tes, &n_tes);
 		blt(tlhs, trhs, n_trhs * sizeof(token_type));
 		n_tlhs = n_trhs;
-		for (j = 0; j < iterations; j++) {
+		for (l = 0; l < iterations; l++) {
 			blt(trhs, tlhs, n_tlhs * sizeof(token_type));
 			n_trhs = n_tlhs;
-			subst_var_with_exp(trhs, &n_trhs, scratch, n, it_v);
+			subst_var_with_exp(trhs, &n_trhs, tes, n_tes, it_v);
 			calc_simp(trhs, &n_trhs);
-			blt(scratch, trhs, n_trhs * sizeof(token_type));
-			n = n_trhs;
+			if (se_compare(trhs, n_trhs, tes, n_tes, &diff_sign) && !diff_sign) {
+				printf(_("Convergence reached after %ld iterations.\n"), l + 1);
+				break;
+			}
+			blt(tes, trhs, n_trhs * sizeof(token_type));
+			n_tes = n_trhs;
 		}
 	} else {
-		simp_side(trhs, &n_trhs);
+		calc_simp(trhs, &n_trhs);
 	}
 	for (j = 0; j < ARR_CNT(sa_mark); j++)
 		sa_mark[j] = false;
@@ -1072,9 +1110,8 @@ char	*cp;
 			k++;
 		}
 	}
-	counter_max = (1L << k) - 1;
-	counter = 0;
-	for (; counter <= counter_max; counter++) {
+	counter_max = (1L << k) - 1L;
+	for (counter = 0; counter <= counter_max; counter++) {
 		blt(tlhs, trhs, n_trhs * sizeof(token_type));
 		n_tlhs = n_trhs;
 		for (j = 0, k = 0; j < ARR_CNT(sa_mark); j++) {
@@ -1170,7 +1207,7 @@ int	*diff_signp;
 
 	rv = se_compare(rhs[i], n_rhs[i], rhs[i], n_rhs[i], diff_signp);
 	if (!rv || *diff_signp) {
-		error(_("Error in compare function or too many terms to compare!"));
+		error(_("Error in compare function or too many terms to compare."));
 		return false;
 	}
 	sign_flag = true;
@@ -1188,7 +1225,7 @@ int	i, j;
 
 	rv = se_compare(lhs[i], n_lhs[i], lhs[i], n_lhs[i], &diff_sign);
 	if (!rv || diff_sign) {
-		error(_("Error in compare function or too many terms to compare!"));
+		error(_("Error in compare function or too many terms to compare."));
 		return false;
 	}
 	sign_flag = true;
@@ -1581,7 +1618,7 @@ char	*cp;
 	}
 	for (; i <= j; i++) {
 		if (n_lhs[i] > 0) {
-			group_sub(i);
+			make_fractions_and_group(i);
 			if (factor_flag || factor_int_flag) {
 				factor_int_sub(i);
 			}
@@ -1672,7 +1709,7 @@ char	*cp;
 			if (int_flag) {
 				uf_repeat_always(rhs[i], &n_rhs[i]);
 			}
-			group_sub(i);
+			make_fractions_and_group(i);
 			if (int_flag) {
 				if (!int_expr(rhs[i], n_rhs[i])) {
 					error(_("Not an integer equation."));
@@ -1746,14 +1783,14 @@ char	*cp;
 	last_v = 0;
 	for (;;) {
 		v = -1;
-		for (j = 0; j < n_lhs[i]; j++) {
+		for (j = 0; j < n_lhs[i]; j += 2) {
 			if (lhs[i][j].kind == VARIABLE) {
 				if (lhs[i][j].token.variable > last_v
 				    && (v == -1 || lhs[i][j].token.variable < v))
 					v = lhs[i][j].token.variable;
 			}
 		}
-		for (j = 0; j < n_rhs[i]; j++) {
+		for (j = 0; j < n_rhs[i]; j += 2) {
 			if (rhs[i][j].kind == VARIABLE) {
 				if (rhs[i][j].token.variable > last_v
 				    && (v == -1 || rhs[i][j].token.variable < v))
@@ -2005,7 +2042,7 @@ char	*cp;
 	if ((rv = setjmp(jmp_save)) != 0) {
 		clean_up();
 		if (rv == 14)
-			error(_("Expression too big."));
+			error(_("Expression too large."));
 		printf(_("Read operation aborted.\n"));
 		goto end_read;
 	}
