@@ -671,7 +671,6 @@ char	*cp;
 	long		v = 0;
 	token_type	solved_v, want;
 	char		*cp_start;
-	int		nl;
 	int		infinity_flag, found_den, complex_den, num, den;
 	int		level;
 
@@ -680,7 +679,7 @@ char	*cp;
 		return false;
 	}
 	if (n_rhs[cur_equation] == 0) {
-/* make expression into equation: */
+/* make expression into an equation: */
 		blt(rhs[cur_equation], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
 		n_rhs[cur_equation] = n_lhs[cur_equation];
 		n_lhs[cur_equation] = 1;
@@ -693,7 +692,6 @@ char	*cp;
 		return false;
 	}
 	solved_v = lhs[cur_equation][0];
-	i = next_espace();
 /* parse the command line or prompt: */
 	if (*cp) {
 		cp = parse_var2(&v, cp);
@@ -719,31 +717,34 @@ char	*cp;
 		if (!case_sensitive_flag) {
 			str_tolower(cp);
 		}
-		if ((cp = parse_section(lhs[i], &nl, cp)) == NULL || nl <= 0) {
+		if ((cp = parse_section(tes, &n_tes, cp)) == NULL || n_tes <= 0) {
 			return false;
 		}
 	} else {
 		list_var(v, 0);
 		snprintf(prompt_str, sizeof(prompt_str), _("as (%s) goes to: "), var_str);
-		if (!get_expr(lhs[i], &nl)) {
+		if (!get_expr(tes, &n_tes)) {
 			return false;
 		}
 	}
 	if (extra_garbage(cp))
 		return false;
-	simp_loop(lhs[i], &nl);
-	infinity_flag = exp_contains_infinity(lhs[i], nl);
+/* copy to a new equation space and work on the copy: */
+	i = next_espace();
+	copy_espace(cur_equation, i);
+	cur_equation = i;
+/* see if the limit expression contains infinity: */
+	simp_loop(tes, &n_tes);
+	infinity_flag = exp_contains_infinity(tes, n_tes);
 /* fully simplify the current equation and group denominators together: */
 	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false);
 	group_proc(rhs[cur_equation], &n_rhs[cur_equation]);
-	found_den = false;
-	complex_den = false;
+	found_den = complex_den = false;
 	for (j = 1; j < n_rhs[cur_equation]; j += 2) {
 		switch (rhs[cur_equation][j].token.operatr) {
 		case DIVIDE:
 		case MODULUS:
-			num = false;
-			den = false;
+			num = den = false;
 			level = rhs[cur_equation][j].level;
 			for (k = j + 2;; k += 2) {
 				if (rhs[cur_equation][k-1].kind == VARIABLE
@@ -778,15 +779,15 @@ char	*cp;
 	}
 	if (!found_den || (infinity_flag && !complex_den)) {
 /* in this case, just substitute the limit variable with the limit expression and simplify: */
-		subst_var_with_exp(rhs[cur_equation], &n_rhs[cur_equation], lhs[i], nl, v);
+		subst_var_with_exp(rhs[cur_equation], &n_rhs[cur_equation], tes, n_tes, v);
 		simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false);
 		return return_result(cur_equation);
 	}
-	if (infinity_flag) {
-/* To take the limit to infinity, */
+	if (n_tes == 1 && tes[0].kind == CONSTANT && tes[0].token.constant == HUGE_VAL) {
+/* To take the limit to positive infinity, */
 /* replace infinity with zero and replace the limit variable with its reciprocal: */
-		nl = 1;
-		lhs[i][0] = zero_token;
+		n_tes = 1;
+		tes[0] = zero_token;
 		tlhs[0] = one_token;
 		tlhs[1].level = 1;
 		tlhs[1].kind = OPERATOR;
@@ -806,8 +807,8 @@ char	*cp;
 		return false;
 	}
 /* replace the limit variable (LHS) with the limit expression: */
-	blt(lhs[cur_equation], lhs[i], nl * sizeof(token_type));
-	n_lhs[cur_equation] = nl;
+	blt(lhs[cur_equation], tes, n_tes * sizeof(token_type));
+	n_lhs[cur_equation] = n_tes;
 /* symbolically simplify the RHS: */
 	symb_flag = true;
 	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false);
