@@ -8,17 +8,19 @@ extern int		n_tokens;
 extern int		n_equations;
 extern int		cur_equation;
 
-extern int		n_lhs[N_EQUATIONS];
-extern int		n_rhs[N_EQUATIONS];
-
 extern token_type	*lhs[N_EQUATIONS];
 extern token_type	*rhs[N_EQUATIONS];
 
-extern int		n_tlhs;
-extern int		n_trhs;
+extern int		n_lhs[N_EQUATIONS];
+extern int		n_rhs[N_EQUATIONS];
 
 extern token_type	*tlhs;
 extern token_type	*trhs;
+extern token_type	*tes;
+
+extern int		n_tlhs;
+extern int		n_trhs;
+extern int		n_tes;
 
 extern token_type	*scratch;
 
@@ -27,21 +29,26 @@ extern token_type	one_token;
 
 extern char		*var_names[MAX_VAR_NAMES];
 
+extern int		precision;
 extern int		case_sensitive_flag;
 extern int		factor_int_flag;
-extern int		groupall;
-extern int		in_calc_cmd;
+extern int		display2d;
+extern int		approximate_roots;
 extern int		preserve_roots;
 extern int		true_modulus;
 extern int		screen_columns;
 extern int		screen_rows;
 extern int		finance_option;
+extern int		autosolve;
+extern int		autocalc;
+extern char		special_variable_character;
 extern int		debug_level;
 extern int		domain_check;
 extern int		color_flag;
 extern int		bold_colors;
 extern int		cur_color;
 extern int		html_flag;
+extern int		readline_enabled;
 extern int		partial_flag;
 extern int		symb_flag;
 extern int		high_prec;
