@@ -10,7 +10,11 @@ def fact(x):
 	if (x == 0):
 		return x + 1
 	d = x
-	while (x > 1):
+	while (x > 2):
 		x -= 1
-		d *= x
+		temp = d * x
+		if (temp <= d):
+			raise ValueError("Factorial result too large.")
+			return
+		d = temp
 	return d
