@@ -42,7 +42,7 @@ double	d1, d2;
 	}
 	d = larger * epsilon;
 	if (d >= divisor) {
-		return 0.0;
+		return 0.0;	/* result would be too inaccurate */
 	}
 	for (count = 1; count < 50; count++) {
 		r1 = fmod(larger, divisor);
@@ -98,8 +98,19 @@ double	*denominatorp;	/* returned denominator */
 
 	*numeratorp = d;
 	*denominatorp = 1.0;
+/* first, see if it is an integer, or very close to an integer: */
 	if (fmod(d, 1.0) == 0.0)
 		return true;
+#if	true	/* set true for more integer oriented math */
+	k3 = fabs(d) * epsilon;
+#else
+	k3 = fabs(d) * small_epsilon;
+#endif
+	k4 = my_round(d);
+	if (k4 != 0.0 && fabs(k4 - d) < k3) {
+		*numeratorp = k4;
+		return true;
+	}
 	if ((divisor = gcd(1.0, d)) > epsilon) {
 		numerator = d / divisor;
 		denominator = 1.0 / divisor;
@@ -111,11 +122,8 @@ double	*denominatorp;	/* returned denominator */
 			return false;
 		if (denominator < 1.5)
 			return false;
-		divisor = gcd(numerator, denominator);	/* make sure the result is a fully reduced fraction */
-		if (fmod(divisor, 1.0) != 0.0) {
-			error("Error in gcd() function!");
-			return false;
-		}
+/* make sure the result is a fully reduced fraction: */
+		divisor = gcd(numerator, denominator);
 		if (divisor > 1.0) {	/* shouldn't happen, but just in case */
 			numerator = numerator / divisor;
 			denominator = denominator / divisor;
@@ -134,10 +142,7 @@ double	*denominatorp;	/* returned denominator */
 
 /*
  * Convert non-integer constants in an equation side to fractions, when appropriate.
- *
- * Return true if equation side was changed.
  */
-int
 make_fractions(equation, np)
 token_type	*equation;	/* equation side pointer */
 int		*np;		/* pointer to length of equation side */
@@ -146,19 +151,21 @@ int		*np;		/* pointer to length of equation side */
 	int	level;
 	double	numerator, denominator;
 	int	inc_level;
-	int	modified = false;
 
 	for (i = 0; i < *np; i += 2) {
 		if (equation[i].kind == CONSTANT) {
 			level = equation[i].level;
 			if (i > 0 && equation[i-1].level == level && equation[i-1].token.operatr == DIVIDE)
 				continue;
-			if (!f_to_fraction(equation[i].token.constant, &numerator, &denominator) || denominator == 1.0)
+			if (!f_to_fraction(equation[i].token.constant, &numerator, &denominator))
 				continue;
+			if (denominator == 1.0) {
+				equation[i].token.constant = numerator;
+				continue;
+			}
 			if ((*np + 2) > n_tokens) {
 				error_huge();
 			}
-			modified = true;
 			inc_level = (*np > 1);
 			if ((i + 1) < *np && equation[i+1].level == level) {
 				switch (equation[i+1].token.operatr) {
@@ -210,5 +217,4 @@ int		*np;		/* pointer to length of equation side */
 			}
 		}
 	}
-	return modified;
 }
