@@ -1,9 +1,9 @@
 /*
- * Mathomatic global variables.
+ * Mathomatic global variables and arrays.
  *
  * Copyright (C) 1987-2006 George Gesslein II.
  *
- * All global variables for Mathomatic are defined here and duplicated in "externs.h".
+ * Most global variables for Mathomatic are defined here and duplicated in "externs.h".
  */
 
 #include "includes.h"
@@ -20,9 +20,11 @@ int		n_rhs[N_EQUATIONS];		/* number of tokens in each rhs[] */
 
 token_type	*tlhs;				/* LHS during solve and temporary storage for expressions */
 token_type	*trhs;				/* RHS during solve and temporary storage for expressions */
+token_type	*tes;				/* temporary equation side, used only in commands */
 
 int		n_tlhs;				/* number of tokens in tlhs */
 int		n_trhs;				/* number of tokens in trhs */
+int		n_tes;				/* number of tokens in tes */
 
 token_type	*scratch;			/* very temporary storage for expressions */
 
@@ -30,15 +32,18 @@ token_type	zero_token;			/* the constant 0.0 as a token */
 token_type	one_token;			/* the constant 1.0 as a token */
 
 /* set options */
+int		precision = 14;			/* the display precision (number of digits) */
 int		case_sensitive_flag = true;	/* "set case_sensitive" flag */
 int		factor_int_flag;		/* factor integers when displaying expressions */
-int		groupall = true;		/* "set display2d" flag */
-int		preserve_roots = true;		/* true if we are to preserve 2^.5, etc. */
-int		in_calc_cmd;			/* true if in calculate command (force approximations) */
+int		display2d = true;		/* "set display2d" flag */
+int		preserve_roots = true;		/* set option to preserve 2^.5, etc. */
 int		true_modulus;			/* true for mathematically correct modulus */
 int		screen_columns = STANDARD_SCREEN_COLUMNS;	/* screen width */
 int		screen_rows = STANDARD_SCREEN_ROWS;		/* screen height */
 int		finance_option;			/* for displaying dollars and cents */
+int		autosolve = true;		/* Allows solving by typing the variable name at the prompt */
+int		autocalc = true;		/* Allows automatically calculating a numerical expression */
+char		special_variable_character = '\\';	/* user defined special character for variable names */
 #if	!SILENT
 int		debug_level;			/* current debug level */
 #endif
@@ -73,13 +78,17 @@ jmp_buf		jmp_save;		/* for setjmp() when an error is encountered */
 int		test_mode;		/* test mode flag (-t) */
 int		quiet_mode;		/* quiet mode (don't display prompts) */
 int		html_flag;		/* true for HTML mode */
+int		readline_enabled = true;
 int		partial_flag;		/* true for partial unfactoring of algebraic fractions */
 int		symb_flag;		/* true for "simplify symbolic" */
 int		high_prec;		/* flag to output constants in higher precision (used when saving equations) */
 int		input_column;		/* current column number on the screen at the beginning of a parse */
 int		sign_flag;		/* true when all "sign" variables are to compare equal */
 int		domain_check;		/* flag to track domain errors in the pow() function */
+int		approximate_roots;	/* true if in calculate command (force approximations) */
 
 /* library variables go here */
+#if	SILENT
 char		*error_str;		/* last error when SILENT is defined */
 char		*result_str;		/* returned result when using as library */
+#endif
