@@ -164,26 +164,30 @@ char	*cp;
 				goto return_ok;
 			}
 #if	!LIBRARY
-			/* the numerical input calculation */
-			for (i = 0; i < n_lhs[n]; i += 2) {
-				if (lhs[n][i].kind == VARIABLE && (lhs[n][i].token.variable & VAR_MASK) > SIGN) {
-					goto return_ok;		/* not numerical (contains a variable) */
+			if (autocalc) {
+				/* the numerical input calculation */
+				for (i = 0; i < n_lhs[n]; i += 2) {
+					if (lhs[n][i].kind == VARIABLE && (lhs[n][i].token.variable & VAR_MASK) > SIGN) {
+						goto return_ok;		/* not numerical (contains a variable) */
+					}
 				}
+				/* make the expression an equation */
+				blt(rhs[n], lhs[n], n_lhs[n] * sizeof(token_type));
+				n_rhs[n] = n_lhs[n];
+				lhs[n][0].level = 1;
+				lhs[n][0].kind = VARIABLE;
+				parse_var(&lhs[n][0].token.variable, "answer");
+				n_lhs[n] = 1;
+				i = cur_equation;
+				/* make the current equation and run the calculate command on it */
+				cur_equation = n;
+				calculate_cmd("");
+				cur_equation = i;
+				/* delete from memory */
+				n_lhs[n] = 0;
+				n_rhs[n] = 0;
+				return true;
 			}
-			/* make the expression an equation */
-			blt(rhs[n], lhs[n], n_lhs[n] * sizeof(token_type));
-			n_rhs[n] = n_lhs[n];
-			lhs[n][0].level = 1;
-			lhs[n][0].kind = VARIABLE;
-			parse_var(&lhs[n][0].token.variable, "answer");
-			n_lhs[n] = 1;
-			i = cur_equation;
-			cur_equation = n;
-			calculate_cmd("");
-			cur_equation = i;
-			n_lhs[n] = 0;
-			n_rhs[n] = 0;
-			return true;
 #endif
 		}
 return_ok:
@@ -382,6 +386,27 @@ char	*cp;	/* pointer to variable name string */
 
 #define P(A)	fprintf(gfp, "%s\n", A)
 
+/*
+ * Output command info and usage.
+ *
+ * Return the number of lines output.
+ */
+int
+display_command(i)
+int	i;	/* command index */
+{
+	int	rows = 3;
+
+	fprintf(gfp, "%s - %s\n", com_list[i].name, com_list[i].info);
+	fprintf(gfp, "Usage: %s %s\n", com_list[i].name, com_list[i].usage);
+	if (com_list[i].secondary_name) {
+		fprintf(gfp, "Alternate name for this command: %s\n", com_list[i].secondary_name);
+		rows++;
+	}
+	fprintf(gfp, "\n");
+	return rows;
+}
+
 /*
  * The help command.
  */
@@ -403,16 +428,23 @@ char	*cp;
 		for (i = 0; i < ARR_CNT(com_list); i++) {
 			if (strncasecmp(cp, com_list[i].name, cp1 - cp) == 0
 			    || (com_list[i].secondary_name && strncasecmp(cp, com_list[i].secondary_name, cp1 - cp) == 0)) {
-				fprintf(gfp, "%s - %s\n", com_list[i].name, com_list[i].info);
-				fprintf(gfp, "Usage: %s %s\n\n", com_list[i].name, com_list[i].usage);
+				display_command(i);
 				flag = true;
 			}
 		}
 		if (flag)
 			return true;
 		if (strncasecmp(cp, "usage", cp1 - cp) == 0) {
-			for (i = 0; i < ARR_CNT(com_list); i++) {
+			P("Command Usage Syntax");
+			P("--------------------");
+			for (i = 0, row = 3; i < ARR_CNT(com_list); i++) {
 				fprintf(gfp, "%s %s\n", com_list[i].name, com_list[i].usage);
+				row++;
+				if (gfp == stdout && screen_rows && row >= (screen_rows - 1)) {
+					row = 1;
+					if (!pause_cmd(""))
+						return false;
+				}
 			}
 			return true;
 		}
@@ -463,17 +495,14 @@ char	*cp;
 			return true;
 		}
 		if (is_all(cp)) {
-			for (i = 0, row = 1;; i++, row += 3) {
-				if (gfp == stdout && screen_rows && row >= (screen_rows - 4)) {
+			for (i = 0, row = 1; i < ARR_CNT(com_list); i++) {
+				row += display_command(i);
+				if (gfp == stdout && screen_rows && row >= (screen_rows - 5)) {
 					row = 1;
 					if (!pause_cmd(""))
 						return false;
 					printf("\n");
 				}
-				if (i >= ARR_CNT(com_list))
-					break;
-				fprintf(gfp, "%s - %s\n", com_list[i].name, com_list[i].info);
-				fprintf(gfp, "Usage: %s %s\n\n", com_list[i].name, com_list[i].usage);
 			}
 			P("End of command list.");
 			return true;
@@ -494,7 +523,7 @@ char	*cp;
 		for (; j > 0; j--)
 			fprintf(gfp, " ");
 	}
-	P("\n\nTo select an equation in memory, just enter the equation number.");
-	P("To solve the current equation, type the variable name at the prompt.");
+	P("\n\nTo select an equation in memory, type the equation number at the main prompt.");
+	P("To solve the current equation, type the variable name at the main prompt.");
 	return true;
 }
