@@ -12,8 +12,6 @@ static int	integrate_sub();
 static int	laplace_sub();
 static int	inv_laplace_sub();
 
-static double	integrate_order;	/* global order of integration */
-
 /*
  * Make variable "v" always raised to a power,
  * unless it is on the right side of a power operator.
@@ -197,15 +195,12 @@ long		v;		/* variable of integration */
 			equation[j].level = level;
 			equation[j].kind = CONSTANT;
 			equation[j].token.constant = 1.0;
-			j++;
 			blt(&equation[eloc+len+1], &equation[eloc], (*np - eloc) * sizeof(token_type));
 			*np += len + 1;
-			k = eloc;
-			equation[k].level = mlevel;
-			equation[k].kind = OPERATOR;
-			equation[k].token.operatr = DIVIDE;
-			k++;
-			blt(&equation[k], &equation[i], len * sizeof(token_type));
+			equation[eloc].level = mlevel;
+			equation[eloc].kind = OPERATOR;
+			equation[eloc].token.operatr = DIVIDE;
+			blt(&equation[eloc+1], &equation[i], len * sizeof(token_type));
 			return true;
 		}
 	}
@@ -214,14 +209,13 @@ long		v;		/* variable of integration */
 	}
 	blt(&equation[eloc+2], &equation[eloc], (*np - eloc) * sizeof(token_type));
 	*np += 2;
-	k = eloc;
-	equation[k].level = mlevel;
-	equation[k].kind = OPERATOR;
-	equation[k].token.operatr = TIMES;
-	k++;
-	equation[k].level = mlevel;
-	equation[k].kind = VARIABLE;
-	equation[k].token.variable = v;
+	equation[eloc].level = mlevel;
+	equation[eloc].kind = OPERATOR;
+	equation[eloc].token.operatr = TIMES;
+	eloc++;
+	equation[eloc].level = mlevel;
+	equation[eloc].kind = VARIABLE;
+	equation[eloc].token.variable = v;
 	return true;
 }
 
@@ -234,12 +228,11 @@ char	*cp;
 {
 	int		i, j;
 	long		v = 0;
-	double		d1;
 	token_type	*source, *dest;
 	int		n1, n2, *nps, *np;
 	int		def_flag;
+	double		d1, integrate_order = 1.0;
 
-	integrate_order = 1.0;
 	if (current_not_defined()) {
 		return false;
 	}
@@ -325,8 +318,7 @@ char	*cp;
 	}
 	*np = n1;
 	cur_equation = i;
-	return_result(cur_equation);
-	return true;
+	return return_result(cur_equation);
 }
 
 /*
@@ -381,7 +373,6 @@ long		v;
 			equation[j].level = level;
 			equation[j].kind = CONSTANT;
 			equation[j].token.constant = -1.0;
-			j++;
 			blt(&equation[eloc+len+3], &equation[eloc], (*np - eloc) * sizeof(token_type));
 			*np += len + 3;
 			k = eloc;
@@ -397,8 +388,7 @@ long		v;
 			k++;
 			equation[k].level = mlevel + 1;
 			equation[k].kind = CONSTANT;
-			equation[k].token.constant = 0.0;
-			k++;
+			equation[k].token.constant = 1.0;
 			return true;
 		}
 	}
@@ -407,14 +397,13 @@ long		v;
 	}
 	blt(&equation[eloc+2], &equation[eloc], (*np - eloc) * sizeof(token_type));
 	*np += 2;
-	k = eloc;
-	equation[k].level = mlevel;
-	equation[k].kind = OPERATOR;
-	equation[k].token.operatr = DIVIDE;
-	k++;
-	equation[k].level = mlevel;
-	equation[k].kind = VARIABLE;
-	equation[k].token.variable = v;
+	equation[eloc].level = mlevel;
+	equation[eloc].kind = OPERATOR;
+	equation[eloc].token.operatr = DIVIDE;
+	eloc++;
+	equation[eloc].level = mlevel;
+	equation[eloc].kind = VARIABLE;
+	equation[eloc].token.variable = v;
 	return true;
 }
 
@@ -464,7 +453,6 @@ long		v;
 			equation[j].level = level;
 			equation[j].kind = CONSTANT;
 			equation[j].token.constant = 1.0;
-			j++;
 			blt(&equation[eloc+len+3], &equation[eloc], (*np - eloc) * sizeof(token_type));
 			*np += len + 3;
 			k = eloc;
@@ -480,8 +468,7 @@ long		v;
 			k++;
 			equation[k].level = mlevel + 1;
 			equation[k].kind = CONSTANT;
-			equation[k].token.constant = 0.0;
-			k++;
+			equation[k].token.constant = 1.0;
 			return true;
 		}
 	}
@@ -540,14 +527,12 @@ char	*cp;
 	blt(dest, source, *nps * sizeof(token_type));
 	n1 = *nps;
 	if (inverse_flag) {
-		if (!poly_in_v(dest, n1, v, true)
-		    || !int_dispatch(dest, &n1, v, inv_laplace_sub)) {
+		if (!poly_in_v(dest, n1, v, true) || !int_dispatch(dest, &n1, v, inv_laplace_sub)) {
 			error(_("Inverse Laplace failed."));
 			return false;
 		}
 	} else {
-		if (!poly_in_v(dest, n1, v, false)
-		    || !int_dispatch(dest, &n1, v, laplace_sub)) {
+		if (!poly_in_v(dest, n1, v, false) || !int_dispatch(dest, &n1, v, laplace_sub)) {
 			error(_("Laplace failed, not a polynomial."));
 			return false;
 		}
@@ -559,8 +544,7 @@ char	*cp;
 	}
 	*np = n1;
 	cur_equation = i;
-	return_result(cur_equation);
-	return true;
+	return return_result(cur_equation);
 }
 
 #if	!LIBRARY
@@ -648,11 +632,11 @@ char	*cp;
 		error(_("Error: Bound contains infinity."));
 		return false;
 	}
-	if ((n_tlhs + 1 + n_trhs + 2) > n_tokens) {
+	if ((n_tlhs + n_trhs + 3) > n_tokens) {
 		error_huge();
 	}
 	if (trap_flag) {
-		debug_string(0, _("Approximating the definite integral using the trapezoidal method..."));
+		debug_string(0, _("Approximating the definite integral using the trapezoid method..."));
 	} else {
 		debug_string(0, _("Approximating the definite integral using Simpson's rule..."));
 	}
@@ -759,11 +743,11 @@ char	*cp;
 			n1 += 2;
 		}
 
-		in_calc_cmd = true;
+		approximate_roots = true;
 		elim_loop(dest, &n1);
 		ufactor(dest, &n1);
 		simp_divide(dest, &n1);
-		in_calc_cmd = false;
+		approximate_roots = false;
 
 		if (exp_contains_infinity(dest, n1)) {
 			error(_("Integration failed because result contains infinity or nan."));
@@ -812,11 +796,11 @@ char	*cp;
 	for (; k < n1; k++)
 		dest[k].level++;
 
-	in_calc_cmd = true;
+	approximate_roots = true;
 	elim_loop(dest, &n1);
 	ufactor(dest, &n1);
 	simp_divide(dest, &n1);
-	in_calc_cmd = false;
+	approximate_roots = false;
 
 	debug_string(0, _("Integration successful."));
 	if (n_rhs[cur_equation]) {
@@ -825,7 +809,6 @@ char	*cp;
 	}
 	*np = n1;
 	cur_equation = i;
-	return_result(cur_equation);
-	return true;
+	return return_result(cur_equation);
 }
 #endif
