@@ -7,17 +7,25 @@
 
 #include "includes.h"
 
+#define	EQUATE_STRING	" = "	/* string displayed between the LHS and RHS of equations */
+
 static int	flist();
 static int	flist_recurse();
 
 /* ANSI color code array */
 static int	carray[] = {
-	32, 33, 31, 35, 34, 36, 37
+	32,	/* green (default color) */
+	33,	/* yellow */
+	31,	/* red */
+	35,	/* magenta */
+	34,	/* blue */
+	36,	/* cyan */
+	37	/* white */
 };
 
 /* HTML color array */
 static char	*html_carray[] = {
-	"#00FF00",
+	"#00FF00",	/* bright green (default color) */
 	"#FFFF00",
 	"#FF9000",
 	"#FF0000",
@@ -26,6 +34,10 @@ static char	*html_carray[] = {
 	"#0000FF"
 };
 
+/* global variables for flist() */
+int	cur_line;	/* current line */
+int	cur_pos;	/* current position in the current line on the screen */
+
 /*
  * Turn color off if color mode is on.
  */
@@ -97,7 +109,7 @@ int	export_flag;	/* true for exportable format (readable by other math programs)
 	}
 	list_proc(lhs[n], n_lhs[n], export_flag);
 	if (n_rhs[n]) {
-		fprintf(gfp, " = ");
+		fprintf(gfp, EQUATE_STRING);
 		list_proc(rhs[n], n_rhs[n], export_flag);
 	}
 	if (export_flag == 1) {
@@ -128,7 +140,7 @@ int	n;	/* equation space number */
 		return false;
 	if (display2d) {
 		/* display in fraction format */
-		group_sub(n);
+		make_fractions_and_group(n);
 		if (factor_int_flag) {
 			factor_int_sub(n);
 		}
@@ -154,7 +166,7 @@ int		n2;
 		printf(_("level %d: "), level);
 		list_proc(p1, n1, false);
 		if (p2 && n2 > 0) {
-			printf(" = ");
+			printf(EQUATE_STRING);
 			list_proc(p2, n2, false);
 		}
 		printf("\n");
@@ -270,13 +282,6 @@ int	lang_code;	/* language code */
 		snprintf(buf, sizeof(buf), "%d", j - 1);
 		strcat(var_str, buf);
 	}
-	if (lang_code > 0) {
-		for (cp = var_str; *cp; cp++) {
-			if (*cp == '\\') {
-				*cp = '_';
-			}
-		}
-	}
 	return(strlen(var_str));
 }
 
@@ -390,7 +395,7 @@ int	export_flag;	/* flag for exportable format (usually false) */
 
 	len = list_string_sub(lhs[n], n_lhs[n], NULL, export_flag);
 	if (n_rhs[n]) {
-		len += 3;
+		len += strlen(EQUATE_STRING);
 		len += list_string_sub(rhs[n], n_rhs[n], NULL, export_flag);
 	}
 	len += 2;
@@ -401,7 +406,7 @@ int	export_flag;	/* flag for exportable format (usually false) */
 	}
 	list_string_sub(lhs[n], n_lhs[n], cp, export_flag);
 	if (n_rhs[n]) {
-		strcat(cp, " = ");
+		strcat(cp, EQUATE_STRING);
 		list_string_sub(rhs[n], n_rhs[n], &cp[strlen(cp)], export_flag);
 	}
 	if (export_flag == 1) {
@@ -450,6 +455,8 @@ int		n;		/* length of expression */
 char		*string;	/* buffer or NULL pointer */
 int		export_flag;
 {
+#define	APPEND(str)	{ if (string) { strcpy(&string[len], str); } len += strlen(str); }
+
 	int	i, j, k, i1;
 	int	min1;
 	int	cur_level;
@@ -466,14 +473,10 @@ int		export_flag;
 		for (i1 = 1; i1 <= k; i1++) {
 			if (j > 0) {
 				cur_level--;
-				if (string)
-					strcat(string, ")");
-				len++;
+				APPEND(")");
 			} else {
 				cur_level++;
-				if (string)
-					strcat(string, "(");
-				len++;
+				APPEND("(");
 			}
 		}
 		switch (equation[i].kind) {
@@ -483,23 +486,20 @@ int		export_flag;
 			}
 			if (high_prec || export_flag) {
 				if (equation[i].token.constant < 0.0) {
-					len += snprintf(buf, sizeof(buf), "(%.20g)", equation[i].token.constant);
+					snprintf(buf, sizeof(buf), "(%.20g)", equation[i].token.constant);
 				} else {
-					len += snprintf(buf, sizeof(buf), "%.20g", equation[i].token.constant);
+					snprintf(buf, sizeof(buf), "%.20g", equation[i].token.constant);
 				}
 			} else if (finance_option) {
-				len += snprintf(buf, sizeof(buf), "%.2f", equation[i].token.constant);
+				snprintf(buf, sizeof(buf), "%.2f", equation[i].token.constant);
 			} else {
-				len += snprintf(buf, sizeof(buf), "%.*g", precision, equation[i].token.constant);
-			}
-			if (string) {
-				strcat(string, buf);
+				snprintf(buf, sizeof(buf), "%.*g", precision, equation[i].token.constant);
 			}
+			APPEND(buf);
 			break;
 		case VARIABLE:
-			len += list_var(equation[i].token.variable, 0 - export_flag);
-			if (string)
-				strcat(string, var_str);
+			list_var(equation[i].token.variable, 0 - export_flag);
+			APPEND(var_str);
 			break;
 		case OPERATOR:
 			cp = _("(unknown operator)");
@@ -527,16 +527,12 @@ int		export_flag;
 				i++;
 				break;
 			}
-			len += strlen(cp);
-			if (string)
-				strcat(string, cp);
+			APPEND(cp);
 			break;
 		}
 	}
 	for (j = cur_level - min1; j > 0; j--) {
-		if (string)
-			strcat(string, ")");
-		len++;
+		APPEND(")");
 	}
 	return len;
 }
@@ -584,7 +580,7 @@ int	int_flag;	/* integer arithmetic flag */
 
 /*
  * Output C, Java, or Python code for an expression.
- * Expression may be modified.
+ * Expression might be modified by this function.
  *
  * Return length of output (number of characters).
  */
@@ -612,8 +608,7 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 		for (i1 = 1; i1 <= k; i1++) {
 			if (j > 0) {
 				cur_level--;
-				fprintf(gfp, ")");
-				len++;
+				len += fprintf(gfp, ")");
 			} else {
 				cur_level++;
 				for (i2 = i + 1; i2 < *np && equation[i2].level >= cur_level; i2 += 2) {
@@ -645,8 +640,7 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 						break;
 					}
 				}
-				fprintf(gfp, "(");
-				len++;
+				len += fprintf(gfp, "(");
 			}
 		}
 		switch (equation[i].kind) {
@@ -715,11 +709,8 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 	return len;
 }
 
-int	cur_line;	/* current line */
-int	cur_pos;	/* current position in the current line on the screen */
-
 /*
- * Display an expression or equation in multi-line fraction format.
+ * Display an equation space in multi-line fraction format.
  * Use color if available.
  *
  * Return true if successful.
@@ -728,7 +719,6 @@ int
 flist_sub(n)
 int	n;	/* equation space number */
 {
-	int	i;
 	int	sind;
 	char	buf[30];
 	int	len, len2, len3;
@@ -745,7 +735,7 @@ int	n;	/* equation space number */
 	sind = n_rhs[n];
 	len += flist(lhs[n], n_lhs[n], false, 0, &max_line, &min_line);
 	if (n_rhs[n]) {
-		len += 3;
+		len += strlen(EQUATE_STRING);
 make_smaller:
 		len2 = flist(rhs[n], sind, false, 0, &high, &low);
 		if (screen_columns && gfp == stdout && (len + len2) >= screen_columns && sind > 0) {
@@ -774,27 +764,25 @@ make_smaller:
 		return list1_sub(n, false);
 	}
 	fprintf(gfp, "\n");
-	for (i = max_line; i >= min_line; i--) {
-		cur_line = i;
-		cur_pos = 0;
-		if (i == 0) {
+	for (cur_line = max_line; cur_line >= min_line; cur_line--) {
+		pos = cur_pos = 0;
+		if (cur_line == 0) {
 			cur_pos += fprintf(gfp, "%s", buf);
 		}
-		pos = strlen(buf);
+		pos += strlen(buf);
 		pos += flist(lhs[n], n_lhs[n], true, pos, &high, &low);
 		if (n_rhs[n]) {
-			if (i == 0) {
-				cur_pos += fprintf(gfp, "%s", " = ");
+			if (cur_line == 0) {
+				cur_pos += fprintf(gfp, "%s", EQUATE_STRING);
 			}
-			pos += 3;
-			flist(rhs[n], sind, true, pos, &high, &low);
+			pos += strlen(EQUATE_STRING);
+			pos += flist(rhs[n], sind, true, pos, &high, &low);
 		}
 		fprintf(gfp, "\n");
 	}
 	if (sind < n_rhs[n]) {
 		fprintf(gfp, "\n");
-		for (i = max2_line; i >= min2_line; i--) {
-			cur_line = i;
+		for (cur_line = max2_line; cur_line >= min2_line; cur_line--) {
 			cur_pos = 0;
 			flist(&rhs[n][sind], n_rhs[n] - sind, true, 0, &high, &low);
 			fprintf(gfp, "\n");
@@ -939,7 +927,11 @@ check_again:
 			if (oflag) {
 				set_color(level-1);
 				for (j = 0; j < len_div; j++) {
-					fprintf(gfp, "-");
+					if (html_flag) {
+						fprintf(gfp, "&mdash;");
+					} else {
+						fprintf(gfp, "-");
+					}
 				}
 				set_color(cur_level-1);
 			}
