@@ -17,7 +17,7 @@ static inline int
 isvarchar(ch)
 int	ch;
 {
-	return(ch == '_' || ch == '\\' || isalpha(ch));
+	return(ch == '_' || ch == special_variable_character || isalpha(ch));
 }
 
 /*
