@@ -261,7 +261,7 @@ do_quick:
 }
 
 /*
- * Compare function for qsort(3).
+ * Compare function for qsort(3) within simpb_side().
  */
 static int
 vcmp(p1, p2)
@@ -288,14 +288,13 @@ sort_type	*p1, *p2;
 /*
  * Beauty simplifier.
  * Neat simplify and factor routine.
- * Factors variables in order.
- * "sign" variables first, then by frequency.
+ * Factors variables in order: "sign" variables first, then by frequency.
  */
 simpb_side(equation, np, power_flag, fc_level)
 token_type	*equation;	/* pointer to the beginning of equation side */
 int		*np;		/* pointer to length of equation side */
-int		power_flag;	/* factor powers flag */
-int		fc_level;	/* factor constants code */
+int		power_flag;	/* factor_power() flag */
+int		fc_level;	/* factor constants code, passed to factor_constants() */
 {
 	int		i;
 	int		vc, cnt;
@@ -374,8 +373,8 @@ token_type	*equation;	/* pointer to the beginning of equation side to simplify *
 int		*np;		/* pointer to length of equation side */
 long		v;		/* variable to factor, 0L or MATCH_ANY to factor all variables */
 double		d;		/* factor expressions raised to the power of this if v */
-int		power_flag;	/* factor power flag */
-int		times_flag;	/* factor times flag */
+int		power_flag;	/* factor_power() flag */
+int		times_flag;	/* factor_times() flag */
 int		fc_level;	/* factor constants code, passed to factor_constants() */
 {
 	do {
@@ -410,7 +409,7 @@ int		*np;		/* pointer to length of equation side */
 	int	i;
 	int	rv = false;
 
-	if (*np == 0)
+	if (*np <= 1)	/* no need to simplify a constant or a variable */
 		return rv;
 	do {
 		do {
@@ -463,7 +462,8 @@ int		*np;
 }
 
 /*
- * Combine all like denominators.  Don't factor times.
+ * Combine all like denominators containing variable "v".
+ * Don't call factor_times().
  */
 simp2_divide(equation, np, v, fc_level)
 token_type	*equation;
@@ -491,8 +491,8 @@ int		fc_level;
  * This is the inner-most loop in Mathomatic, make it fast.
  */
 organize(equation, np)
-token_type	*equation;
-int		*np;
+token_type	*equation;	/* equation side pointer */
+int		*np;		/* pointer to length of equation side */
 {
 	if (*np <= 0 || (*np & 1) != 1) {
 		error(_("Organize called with bad expression size!"));
@@ -538,12 +538,14 @@ int		level, invert;
 }
 
 /*
- * Recursively organize an expression at "level" of parentheses.
+ * Recurse through every sub-expression in "equation", starting at "loc",
+ * moving up levels to "level" of parentheses.
  */
 static int
 org_recurse(equation, np, loc, level, elocp)
-token_type	*equation;
-int		*np, loc, level, *elocp;
+token_type	*equation;	/* equation side pointer */
+int		*np;		/* pointer to length of equation side */
+int		loc, level, *elocp;
 {
 	token_type	*p1, *bp, *ep;
 	int		op, sub_op;
@@ -1449,6 +1451,7 @@ int		n2, l2, *diff_signp;
 	int		last_op1, op1, op2;
 	int		diff_op;
 	double		d1, c1, c2;
+	double		compare_epsilon = epsilon;
 
 	*diff_signp = false;
 	if (n1 == 1 && n2 == 1) {
@@ -1472,7 +1475,7 @@ int		n2, l2, *diff_signp;
 				*diff_signp = true;
 				return true;
 			}
-			d1 = fabs(c1) * epsilon;
+			d1 = fabs(c1) * compare_epsilon;
 			if (fabs(c1 - c2) < d1) {
 				return true;
 			}
@@ -1528,7 +1531,7 @@ int		n2, l2, *diff_signp;
 	}
 	if (diff_op) {
 		if (p1->kind == CONSTANT && p1->level == l1 && op1 == TIMES) {
-			if (fabs(fabs(p1->token.constant) - 1.0) <= epsilon) {
+			if (fabs(fabs(p1->token.constant) - 1.0) <= compare_epsilon) {
 				if (!compare_recurse(p1 + 2, n1 - 2, min_level(p1 + 2, n1 - 2), p2, n2, l2, diff_signp)) {
 					return false;
 				}
@@ -1539,7 +1542,7 @@ int		n2, l2, *diff_signp;
 			}
 		}
 		if (p2->kind == CONSTANT && p2->level == l2 && op2 == TIMES) {
-			if (fabs(fabs(p2->token.constant) - 1.0) <= epsilon) {
+			if (fabs(fabs(p2->token.constant) - 1.0) <= compare_epsilon) {
 				if (!compare_recurse(p1, n1, l1, p2 + 2, n2 - 2, min_level(p2 + 2, n2 - 2), diff_signp)) {
 					return false;
 				}
@@ -1573,13 +1576,11 @@ no_op2:
 				break;
 		for (i = 0;; i++) {
 			if (i >= oc2) {
-				if ((op1 == TIMES || op1 == DIVIDE)
-				    && pv1->level == l1 && pv1->kind == CONSTANT) {
-					if (fabs(pv1->token.constant + 1.0) <= epsilon) {
-						*diff_signp ^= true;
-						break;
-					}
-					if (fabs(pv1->token.constant - 1.0) <= epsilon) {
+				if ((op1 == TIMES || op1 == DIVIDE) && pv1->level == l1 && pv1->kind == CONSTANT) {
+					if (fabs(fabs(pv1->token.constant) - 1.0) <= compare_epsilon) {
+						if (pv1->token.constant < 0.0) {
+							*diff_signp ^= true;
+						}
 						break;
 					}
 				}
@@ -1643,13 +1644,11 @@ no_op2:
 	}
 	for (i = 0; i < oc2; i++) {
 		if (!used[i]) {
-			if ((op2 == TIMES || op2 == DIVIDE)
-			    && opa2[i]->level == l2 && opa2[i]->kind == CONSTANT) {
-				if (fabs(opa2[i]->token.constant + 1.0) <= epsilon) {
-					*diff_signp ^= true;
-					continue;
-				}
-				if (fabs(opa2[i]->token.constant - 1.0) <= epsilon) {
+			if ((op2 == TIMES || op2 == DIVIDE) && opa2[i]->level == l2 && opa2[i]->kind == CONSTANT) {
+				if (fabs(fabs(opa2[i]->token.constant) - 1.0) <= compare_epsilon) {
+					if (opa2[i]->token.constant < 0.0) {
+						*diff_signp ^= true;
+					}
 					continue;
 				}
 			}
