@@ -35,7 +35,7 @@ int	have;	/* equation number to solve */
 	n_lhs[want] = 0;
 	n_rhs[want] = 0;
 #if	!SILENT
-	if (!rv) {
+	if (rv <= 0) {
 		printf(_("Solve failed.\n"));
 	}
 #endif
@@ -75,10 +75,10 @@ int		*rightnp;	/* size of RHS */
 	token_type	*p1, *b1, *ep;
 	long		v;
 	int		need_flip;
-	int		uf_flag = false;
+	int		uf_flag = false;	/* unfactor flag */
 	int		qtries = 0;
-	int		zflag;
-	int		zsolve;
+	int		zflag;			/* true if RHS is currently zero */
+	int		zsolve;			/* true if we are solving for zero */
 	int		inc_count = 0;
 	int		zero_solved = false;
 	double		numerator, denominator;
@@ -113,11 +113,10 @@ int		*rightnp;	/* size of RHS */
 		}
 		zsolve = true;
 	}
-#if	true
 	uf_power(tlhs, &n_tlhs);
 	uf_power(trhs, &n_trhs);
-#endif
 simp_again:
+	/* Make sure equation is a bit simplified. */
 	list_tdebug(2);
 	simps_side(tlhs, &n_tlhs, zsolve);
 	if (uf_flag) {
@@ -129,7 +128,7 @@ simp_again:
 	}
 	list_tdebug(1);
 no_simp:
-	/* Move from the RHS to the LHS. */
+	/* First selectively move subexpressions from the RHS to the LHS. */
 	op = 0;
 	ep = &trhs[n_trhs];
 	if (zsolve) {
@@ -194,7 +193,7 @@ no_simp:
 		simps_side(trhs, &n_trhs, zsolve);
 	}
 left_again:
-	worked = 1;
+	worked = true;
 	uf_flag = false;
 see_work:
 	/* See if we have solved the equation. */
@@ -349,8 +348,7 @@ zero_simp:
 			if (p1 >= ep) {
 				if (found_count == 0) { /* if solve variable no longer in LHS */
 					for (i = 0; i < n_trhs; i += 2) {
-						if (trhs[i].kind == VARIABLE
-						    && trhs[i].token.variable == v) {
+						if (trhs[i].kind == VARIABLE && trhs[i].token.variable == v) {
 							/* solve variable in RHS */
 							return false;
 						}
@@ -387,11 +385,11 @@ zero_simp:
 					return -2;
 				}
 				zflag = (n_trhs == 1 && trhs[0].kind == CONSTANT && trhs[0].token.constant == 0.0);
+				if (zflag) {
+					/* overwrite -0.0 */
+					trhs[0].token.constant = 0.0;
+				}
 				if (need_flip >= found_count) {
-					if (zflag) {
-						/* overwrite -0.0 */
-						trhs[0].token.constant = 0.0;
-					}
 					if (!flip(tlhs, &n_tlhs, trhs, &n_trhs))
 						return false;
 					list_tdebug(2);
@@ -401,7 +399,7 @@ zero_simp:
 					goto left_again;
 				}
 				if (worked && !uf_flag) {
-					worked--;
+					worked = false;
 					debug_string(1, _("Unfactoring..."));
 					partial_flag = false;
 					uf_simp(tlhs, &n_tlhs);
@@ -488,14 +486,15 @@ zero_simp:
 						    || fabs(numerator) != 1.0 || denominator < 2.0) {
 							continue;
 						}
-						for (j = i - 1; (j >= 0) && tlhs[j].level >= tlhs[i].level; j--) {
+						for (j = i - 1; j >= 0 && tlhs[j].level >= tlhs[i].level; j--) {
 							if (tlhs[j].kind == VARIABLE && tlhs[j].token.variable == v) {
 								if (b1) {
 									if (fabs(b1->token.constant) < fabs(tlhs[i+1].token.constant)) {
 										b1 = &tlhs[i+1];
 									}
-								} else
+								} else {
 									b1 = &tlhs[i+1];
+								}
 								break;
 							}
 						}
@@ -529,9 +528,8 @@ zero_simp:
 					return false;
 				if (zero_solved) {
 					qtries++;
-				} else {
-					zero_solved = true;
 				}
+				zero_solved = true;
 				if (quad_solve(v)) {
 					goto left_again;
 				} else {
@@ -703,7 +701,9 @@ long	v;	/* solve variable */
 	double		high_power = 0.0;
 
 	uf_simp(trhs, &n_trhs);
-	factorv(trhs, &n_trhs, v);
+	while (factor_plus(trhs, &n_trhs, v, 0.0)) {
+		simp_loop(trhs, &n_trhs);
+	}
 	list_tdebug(1);
 
 	found = false;
@@ -994,7 +994,7 @@ big_bbreak:
 	} else if (high_power == 4.0) {
 		printf(_("Equation was biquadratic.\n"));
 	} else {
-		printf(_("Equation was a degree %g quadratic.\n"), high_power);
+		printf(_("Equation was a degree %.*g quadratic.\n"), precision, high_power);
 	}
 #endif
 	return true;
@@ -1019,10 +1019,8 @@ int		*side1np;	/* pointer to the length of "side1p" */
 token_type	*side2p;	/* equation side pointer */
 int		*side2np;	/* pointer to the length of "side2p" */
 {
-	int		i;
 	token_type	*p1, *p2, *ep;
-	int		oldn;
-	int		operandn;
+	int		oldn, operandn;
 	double		numerator, denominator;
 	static int	prev_n1, prev_n2;
 	double		d1, d2;
@@ -1047,17 +1045,6 @@ int		*side2np;	/* pointer to the length of "side2p" */
 	case POWER:
 	case MODULUS:
 		break;
-	case FACTORIAL:
-		i = oldn - 2;
-		if (i >= 0 && side1p[i].level == 1 && side1p[i].token.operatr == op) {
-			i = *side2np - 2;
-			if (i >= 0 && side2p[i].level == 1 && side2p[i].token.operatr == op) {
-				debug_string(1, _("Removing factorial from both sides of the equation:"));
-				*side1np -= 2;
-				*side2np -= 2;
-				return true;
-			}
-		}
 	default:
 		return false;
 	}
@@ -1150,7 +1137,7 @@ int		*side2np;	/* pointer to the length of "side2p" */
 		p2++;
 		p2->level = 2;
 		p2->kind = VARIABLE;
-		p2->token.variable = V_INTEGER;
+		parse_var(&p2->token.variable, "integer");
 		p2++;
 		p2->level = 2;
 		p2->kind = OPERATOR;
