@@ -27,8 +27,8 @@ static void	group_recurse();
  */
 int
 super_factor(equation, np, start_flag)
-token_type	*equation;
-int		*np;
+token_type	*equation;	/* pointer to the beginning of the equation side */
+int		*np;		/* pointer to the length of the equation side */
 int		start_flag;
 {
 	int	rv;
@@ -339,7 +339,7 @@ do_gcd:
 /*
  * This function is the guts of the "flist" command.
  */
-group_sub(n)
+make_fractions_and_group(n)
 int	n;
 {
 	if (n_lhs[n] <= 0)
